#ifndef DRIVE_H
#define DRIVE_H

#include <stdint.h>

typedef uint8_t drive_file_id_t;

typedef uint8_t fs_id_t;
typedef uint8_t drive_id_t;

#define MAX_DRIVES 256

struct drive {
  char *drive_type;
  char *fs_type;

  uint8_t (*read_sectors)(const struct drive *d, uint32_t start, uint32_t count, void *buffer);
  uint8_t (*write_sectors)(const struct drive *d, uint32_t start, uint32_t count, const void *buffer);
  uint32_t n_sectors;
  drive_id_t drive_id;

  drive_file_id_t (*get_file)(const struct drive *d, const char *path);
  void (*free_file)(const struct drive *d, drive_file_id_t fid);
  void (*load_sector)(const struct drive *d, drive_file_id_t fid, uint32_t sector, void *at);
  uint32_t (*get_free_sectors)(const struct drive *d);
  fs_id_t fs_id;
};

extern uint8_t n_drives;
extern struct drive drives[MAX_DRIVES];

void commit_drive(struct drive data);

#endif